@@ -28,10 +28,11 @@ class ViessmannProtocol(asyncio.Protocol):
         self._buf = b''
         self._lock = None
         self._viessmann_lock = None
+        self._read_lock = None
         self._ack_event = asyncio.Event()
         self._periodic_event = asyncio.Event()
-        self._result_future = None
-        self.sent_pkt = None
+        self._sent_pkt = None
+        self._recv_future = None
 
     def data_received(self, data: bytes):
         self._transport.pause_reading()
@@ -64,16 +65,16 @@ class ViessmannProtocol(asyncio.Protocol):
                 except:
                     pkt = Raw(msg_bytes)
 
-                if (not self._result_future) or \
-                   self._result_future.done() or \
-                   self._result_future.cancelled():
+                if not self._recv_future or \
+                   self._recv_future.done() or \
+                   self._recv_future.cancelled():
                     continue
 
                 if pkt.answers(self._sent_pkt):
-                    self._result_future.set_result(pkt)
+                    self._recv_future.set_result(pkt)
                 else:
                     logger.error("%s does not answer %s", pkt.show(dump=True), self._sent_pkt.show(dump=True))
-                    self._result_future.cancel()
+                    self._recv_future.cancel()
             else:
                 logger.error("Unknown byte: %02x" % (self._buf[0]))
                 self._buf = self._buf[1:]
@@ -86,17 +87,19 @@ class ViessmannProtocol(asyncio.Protocol):
         self._ack_event.clear()
 
     async def send(self, pkt, timeout=3):
-        if not self._connected:
-            logger.debug("Reconnecting...")
-            await self._reconnect_viessmann()
-        await self._send_ack(bytes(pkt), timeout)
+        async with self._read_lock:
+            if not self._connected:
+                await self._reconnect_viessmann()
+            await self._send_ack(bytes(pkt), timeout)
 
     async def send_recv(self, pkt, timeout=3):
-        self._sent_pkt = pkt
-        self._result_future = self._loop.create_future()
-        await self.send(pkt, timeout)
-        answer = await asyncio.wait_for(self._result_future, timeout)
-        return answer
+        async with self._read_lock:
+            self._sent_pkt = pkt
+            self._recv_future = self._loop.create_future()
+            if not self._connected:
+                await self._reconnect_viessmann()
+            await self._send_ack(bytes(pkt), timeout)
+            return await asyncio.wait_for(self._recv_future, timeout)
 
     async def readdata(self, address, length):
         request = VS2Header() / VS2Data(type="request", command="readdata", address=address, data_len=length)
@@ -107,10 +110,13 @@ class ViessmannProtocol(asyncio.Protocol):
             raise Exception("error reading data: %s" % (response.show(dump=True)))
 
     async def writedata(self, address, data):
-        equest = VS2Header() / VS2Data(type="request", command="writedata", address=address, data_len=len(data), data=data)
-        await self.send(request)
+        request = VS2Header() / VS2Data(type="request", command="writedata", address=address, data_len=len(data), data=data)
+        response = await self.send_recv(request)
+        if response.type != 0x01: # response
+            raise Exception("error writing data: %s" % (response.show(dump=True)))
 
     async def _reconnect_viessmann(self):
+        logger.debug("Viessmann reconnecting...")
         self._periodic_event.clear()
         async with self._viessmann_lock:
             self._transport.write(self._END_COMMUNICATION)
@@ -162,6 +168,7 @@ class ViessmannProtocol(asyncio.Protocol):
         self._loop = loop
         self._lock = asyncio.Lock(loop=loop)
         self._viessmann_lock = asyncio.Lock(loop=loop)
+        self._read_lock = asyncio.Lock(loop=loop)
         self._running = True
         await self._reconnect(delay=0)
 
